@@ -1,8 +1,8 @@
 /**************************************************************************
 **       Title: Turn a Digital Camera into a V4L device using vloopback
 **    $RCSfile: dc1394_vloopback.c,v $
-**   $Revision: 1.10 $$Name:  $
-**       $Date: 2004/03/09 08:41:44 $
+**   $Revision: 1.11 $$Name:  $
+**       $Date: 2004/08/10 07:57:22 $
 **   Copyright: LGPL $Author: ddouxchamps $
 ** Description:
 **
@@ -21,6 +21,9 @@
 **-------------------------------------------------------------------------
 **
 **  $Log: dc1394_vloopback.c,v $
+**  Revision 1.11  2004/08/10 07:57:22  ddouxchamps
+**  Removed extra buffering (Johann Schoonees)
+**
 **  Revision 1.10  2004/03/09 08:41:44  ddouxchamps
 **  patch from Johann Schoonees for extra buffering
 **
@@ -554,7 +557,7 @@ int dc_start(int palette)
 	 
 	if (dc1394_dma_setup_capture(handle, camera.node,  channel,
 							FORMAT_VGA_NONCOMPRESSED, mode,
-							speed, FRAMERATE_15, DC1394_BUFFERS, 1, DROP_FRAMES,
+							speed, FRAMERATE_15, DC1394_BUFFERS, DROP_FRAMES,
 							dc_dev_name, &camera) != DC1394_SUCCESS) 
 	{
 		fprintf(stderr, "unable to setup camera- check line %d of %s to make sure\n",
