@@ -80,7 +80,7 @@ _dc1394_video_iso_handler(raw1394handle_t handle,
 #ifdef LIBRAW1394_OLD
     if ( (data[0] & 0x1) && (_dc1394_frame_captured[channel] != 1) )
 #else
-    if ( (ntohl(data[0]) & 0x01000000UL) && (_dc1394_frame_captured[channel] != 1) )
+    if ( (ntohl(data[0]) & 0x00000001UL) && (_dc1394_frame_captured[channel] != 1) )
 #endif
 
     {
@@ -321,26 +321,8 @@ _dc1394_dma_basic_setup(int channel,
     }
 
     camera->dma_buffer_size= vmmap.buf_size * vmmap.nb_buffers;
-
-    /* allocate extra buffers to properly multiplex cameras */
-    camera->dma_extra_count = 0;
     camera->num_dma_buffers_behind = 0;
-	if (camera->do_extra_buffering)
-	{
-		camera->dma_extra_buffer = (unsigned char *) malloc(camera->dma_buffer_size*sizeof(unsigned char));
-		if (camera->dma_extra_buffer == NULL)
-		{
-			printf("(%s) failed to allocate internal buffers!\n", __FILE__);
-			ioctl(camera->dma_fd, VIDEO1394_IOC_UNLISTEN_CHANNEL, &vmmap.channel);
-			munmap((void*)camera->dma_ring_buffer,camera->dma_buffer_size);
-			return DC1394_FAILURE;
-		}
-	}	
-	else
-	{
-		camera->dma_extra_buffer = NULL;
-	}
-	return DC1394_SUCCESS;
+    return DC1394_SUCCESS;
 }
 
 
@@ -529,7 +511,6 @@ dc1394_dma_setup_capture(raw1394handle_t handle, nodeid_t node,
                          int channel, int format, int mode,
                          int speed, int frame_rate,
                          int num_dma_buffers, 
-			 int do_extra_buffering,
 			 int drop_frames,
 			 const char *dma_device_file,
 			 dc1394_cameracapture *camera)
@@ -547,7 +528,6 @@ dc1394_dma_setup_capture(raw1394handle_t handle, nodeid_t node,
 					       QUERY_FROM_CAMERA, /*width*/
 					       QUERY_FROM_CAMERA, /*height*/
 					       num_dma_buffers,
-					       do_extra_buffering,
 					       drop_frames,
 					       dma_device_file,
 					       camera);  
@@ -555,7 +535,6 @@ dc1394_dma_setup_capture(raw1394handle_t handle, nodeid_t node,
     else {
       camera->port = camera_handle->port;
       camera->dma_device_file = dma_device_file;
-      camera->do_extra_buffering = do_extra_buffering;
       camera->drop_frames = drop_frames;
 
       if (_dc1394_basic_setup(handle,node, channel, format, mode,
@@ -598,12 +577,6 @@ dc1394_dma_release_camera(raw1394handle_t handle,
 	    
 	  }
     }
-    
-    if (camera->dma_extra_buffer != NULL)
-    {
-        free( camera->dma_extra_buffer);
-    }
-
 
     return DC1394_SUCCESS;
 }
@@ -657,8 +630,6 @@ _dc1394_dma_multi_capture_private(dc1394_cameracapture *cams, int num, dc1394vid
         cb = (cams[i].dma_last_buffer + 1) % cams[i].num_dma_buffers;
         cams[i].dma_last_buffer = cb;
 
-		if (cams[i].dma_extra_count == 0)
-		{
 			vwait.channel = cams[i].channel;
 			vwait.buffer = cb;
 			switch (policy) {
@@ -689,22 +660,6 @@ _dc1394_dma_multi_capture_private(dc1394_cameracapture *cams, int num, dc1394vid
 
 			extra_buf = vwait.buffer;
 
-			if (cams[i].do_extra_buffering)
-			{
-				/* get the number of buffers by which we are behind */
-				cams[i].dma_extra_count = vwait.buffer;
-				extra_buf = 0; 	/* do_extra_buffering has precedence over drop_frames.*/
-	
-				/* copy the extra frames into internal buffer */
-				for (j = cams[i].dma_extra_count-1; j >= 0; j--)
-				{
-					cb = (cb+1) % cams[i].num_dma_buffers;
-					memcpy( (cams[i].dma_extra_buffer + j * cams[i].dma_frame_size),
-							(cams[i].dma_ring_buffer + cb * cams[i].dma_frame_size),
-							cams[i].dma_frame_size );
-				}
-			}
-
 			if (cams[i].drop_frames)
 			{
 				if (extra_buf > 0)
@@ -740,17 +695,6 @@ _dc1394_dma_multi_capture_private(dc1394_cameracapture *cams, int num, dc1394vid
 
 			cams[i].filltime = vwait.filltime;
 			cams[i].num_dma_buffers_behind = extra_buf;
-		}
-		else 
-		{
-            /* point to an internal extra buffer */
-		    /* Note that filltimes have been lost.*/
-            cams[i].capture_buffer = (int*)(cams[i].dma_extra_buffer + 
-                                            (cams[i].dma_extra_count-1) *
-                                            cams[i].dma_frame_size);
-	    cams[i].dma_extra_count--;
-        }
-	
     }
 
     return DC1394_SUCCESS;
