@@ -23,6 +23,9 @@
  * Dan Dennedy <dan@dennedy.org>
  *  - bug fixes
  *
+ * Yves Jaeger <yves.jaeger@fraenz-jaeger.de>
+ *  - software trigger functions
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -87,6 +90,8 @@
 #define REG_CAMERA_ONE_SHOT            0x61CU
 #define REG_CAMERA_MEM_SAVE_CH         0x620U
 #define REG_CAMERA_CUR_MEM_CH          0x624U
+#define REG_CAMERA_SOFT_TRIGGER        0x62CU
+#define REG_CAMERA_DATA_DEPTH          0x630U
 
 #define REG_CAMERA_FEATURE_HI_BASE     0x800U
 #define REG_CAMERA_FEATURE_LO_BASE     0x880U
@@ -104,6 +109,9 @@
 #define REG_CAMERA_FOCUS               0x828U
 #define REG_CAMERA_TEMPERATURE         0x82CU
 #define REG_CAMERA_TRIGGER_MODE        0x830U
+#define REG_CAMERA_TRIGGER_DELAY       0x834U
+#define REG_CAMERA_WHITE_SHADING       0x838U
+#define REG_CAMERA_FRAME_RATE_FEATURE  0x83CU
 #define REG_CAMERA_ZOOM                0x880U
 #define REG_CAMERA_PAN                 0x884U
 #define REG_CAMERA_TILT                0x888U
@@ -199,41 +207,41 @@ const char *dc1394_feature_desc[NUM_FEATURES] =
   are in a packet given a mode and a frame rate
   This is defined in the 1394 digital camera spec 
 */
-const int quadlets_per_packet_format_0[42] = 
+const int quadlets_per_packet_format_0[56] = 
 {
-     -1,  -1,  15,  30,  60,  -1,
-     -1,  20,  40,  80, 160,  -1,
-     -1,  60, 120, 240, 480,  -1,
-     -1,  80, 160, 320, 640,  -1,
-     -1, 120, 240, 480, 960,  -1,
-     -1,  40,  80, 160, 320, 640,
-     -1,  80, 160, 320, 640,  -1
+  -1,  -1,  15,  30,  60,  120,  240,  480,
+  10,  20,  40,  80, 160,  320,  640, 1280,
+  30,  60, 120, 240, 480,  960, 1920, 3840,
+  40,  80, 160, 320, 640, 1280, 2560, 5120,
+  60, 120, 240, 480, 960, 1920, 3840, 7680,
+  20,  40,  80, 160, 320,  640, 1280, 2560,
+  40,  80, 160, 320, 640, 1280, 2560, 5120
 };
-
-const int quadlets_per_packet_format_1[48] = 
+ 
+const int quadlets_per_packet_format_1[64] = 
 {
-     -1, 125, 250, 500, 1000,   -1,
-     -1,  -1, 375, 750,   -1,   -1,
-     -1,  -1, 125, 250,  500, 1000,
-     96, 192, 384, 768,   -1,   -1,
-    144, 288, 576,  -1,   -1,   -1,
-     48,  96, 192, 384,  768,   -1,
-     -1, 125, 250, 500, 1000,   -1,
-     96, 192, 384, 768,   -1,   -1
+   -1, 125, 250,  500, 1000, 2000, 4000, 8000,
+   -1,  -1, 375,  750, 1500, 3000, 6000,   -1,
+   -1,  -1, 125,  250,  500, 1000, 2000, 4000,
+   96, 192, 384,  768, 1536, 3072, 6144,   -1,
+  144, 288, 576, 1152, 2304, 4608,   -1,   -1,
+   48,  96, 192,  384,  768, 1536, 3073, 6144,
+   -1, 125, 250,  500, 1000, 2000, 4000, 8000,
+   96, 192, 384,  768, 1536, 3072, 6144,   -1
 };
-
-const int quadlets_per_packet_format_2[48] = 
+ 
+const int quadlets_per_packet_format_2[64] = 
 {
-    160, 320,  640,   -1, -1, -1,
-    240, 480,  960,   -1, -1, -1,
-     80, 160,  320,  640, -1, -1,
-    250, 500, 1000,   -1, -1, -1,
-    375, 750,   -1,   -1, -1, -1,
-    125, 250,  500, 1000, -1, -1,
-    160, 320,  640,   -1, -1, -1,
-    250, 500, 1000,   -1, -1, -1
+  160, 320,  640, 1280, 2560, 5120,   -1,  -1,
+  240, 480,  960, 1920, 3840, 7680,   -1,  -1,
+   80, 160,  320,  640, 1280, 2560, 5120,  -1,
+  250, 500, 1000, 2000, 4000, 8000,   -1,  -1,
+  375, 750, 1500, 3000, 6000,   -1,   -1,  -1,
+  125, 250,  500, 1000, 2000, 4000, 8000,  -1,
+  160, 320,  640, 1280, 2560, 5120,   -1,  -1,
+  250, 500, 1000, 2000, 4000, 8000,   -1,  -1
 };
-  
+   
 
 /**********************/
 /* Internal functions */
@@ -325,7 +333,7 @@ _dc1394_get_quadlets_per_packet(int format, int mode, int frame_rate)
         if ( ((mode >= MODE_FORMAT0_MIN) && (mode <= MODE_FORMAT0_MAX)) && 
              ((frame_rate >= FRAMERATE_MIN) && (frame_rate <= FRAMERATE_MAX)) )
         {
-            return quadlets_per_packet_format_0[6*mode_index+frame_rate_index];
+            return quadlets_per_packet_format_0[NUM_FRAMERATES*mode_index+frame_rate_index];
         }
         else
         {
@@ -340,7 +348,7 @@ _dc1394_get_quadlets_per_packet(int format, int mode, int frame_rate)
         if ( ((mode >= MODE_FORMAT1_MIN) && (mode <= MODE_FORMAT1_MAX)) && 
              ((frame_rate >= FRAMERATE_MIN) && (frame_rate <= FRAMERATE_MAX)) )
         {
-            return quadlets_per_packet_format_1[6*mode_index+frame_rate_index];
+            return quadlets_per_packet_format_1[NUM_FRAMERATES*mode_index+frame_rate_index];
         }
         else
         {
@@ -355,7 +363,7 @@ _dc1394_get_quadlets_per_packet(int format, int mode, int frame_rate)
         if ( ((mode >= MODE_FORMAT2_MIN) && (mode <= MODE_FORMAT2_MAX)) && 
              ((frame_rate >= FRAMERATE_MIN) && (frame_rate <= FRAMERATE_MAX)) )
         {
-            return quadlets_per_packet_format_2[6*mode_index+frame_rate_index];
+            return quadlets_per_packet_format_2[NUM_FRAMERATES*mode_index+frame_rate_index];
         }
         else
         {
@@ -1041,43 +1049,126 @@ dc1394_is_camera(raw1394handle_t handle, nodeid_t node, dc1394bool_t *value)
 }
 
 int
-dc1394_get_sw_version(raw1394handle_t handle, nodeid_t node, quadlet_t *value)
+dc1394_get_sw_version(raw1394handle_t handle, nodeid_t node, int *version)
 {
-    dc1394_camerahandle *camera;
-    camera = (dc1394_camerahandle*) raw1394_get_userdata( handle );
+  dc1394_camerahandle *camera;
+  octlet_t offset;
+  octlet_t ud_offset = 0;
+  octlet_t udd_offset = 0;
+  quadlet_t quadval = 0;
+
+  camera = (dc1394_camerahandle*) raw1394_get_userdata( handle );  
+  if (camera != NULL && camera->sw_version > 0) {
+    *version = camera->sw_version;
+  }
+  else {
     
-    if (camera != NULL && camera->sw_version != 0)
-    {
-        *value = camera->sw_version;
+    /* get the unit_directory offset */
+    offset= ROM_ROOT_DIRECTORY;
+    if (GetConfigROMTaggedRegister(handle, node, 0xD1, &offset, &quadval)!=DC1394_SUCCESS) {
+      *version= -1;
+      return DC1394_FAILURE;
+    }
+    else {
+      ud_offset=(quadval & 0xFFFFFFUL)*4+offset;
     }
-    else
-    {
-        octlet_t offset;
-        octlet_t ud_offset = 0;
-        quadlet_t quadval = 0;
-    
-        /* get the unit_directory offset */
-        offset= ROM_ROOT_DIRECTORY;
-        if (GetConfigROMTaggedRegister(handle, node, 0xD1, &offset, &quadval)!=DC1394_SUCCESS) {
-          *value= DC1394_FALSE;
-          return DC1394_FAILURE;
-        }
-        else {
-          ud_offset=(quadval & 0xFFFFFFUL)*4+offset;
-        }
       
-        /* get the unit_sw_version  */
-        offset = ud_offset;
-        if (GetConfigROMTaggedRegister(handle, node, 0x13, &offset, &quadval)!=DC1394_SUCCESS) {
-          *value= DC1394_FALSE;
-          return DC1394_FAILURE;
-        }
-        else {
-          *value=quadval&0xFFFFFFUL;
-	  camera->sw_version=*value;
-        }
+    /* get the unit_sw_version  */
+    offset = ud_offset;
+    if (GetConfigROMTaggedRegister(handle, node, 0x13, &offset, &quadval)!=DC1394_SUCCESS) {
+      *version = -1;
+      return DC1394_FAILURE;
     }
-    return DC1394_SUCCESS;
+    else {
+      switch (quadval&0xFFFFFFUL) {
+      case 0x100:
+	*version=IIDC_VERSION_1_04;
+	break;
+      case 0x101:
+	*version=IIDC_VERSION_1_20;
+	break;
+      case 0x102:
+	*version=IIDC_VERSION_1_30;
+	break;
+      case 0x114:
+      case 0x800002:
+	/* get the unit_spec_ID (should be 0x00A02D for 1394 digital camera) */
+	if (GetConfigROMTaggedRegister(handle, node, 0x12, &ud_offset, &quadval)!=DC1394_SUCCESS) {
+	  *version = -1;
+	  return DC1394_FAILURE;
+	}
+	else {
+	  quadval&=0xFFFFFFUL;
+	}
+	if (quadval == 0x00B09DUL)
+	  *version=IIDC_VERSION_PTGREY;
+	else
+	  *version = -1;
+	break;
+      }
+    }
+  }
+  //fprintf(stderr,"sw version is %d\n",*version);
+  /*IIDC 1.31 check*/
+  if (*version==IIDC_VERSION_1_30) {
+    
+    /* get the unit_dependent_directory offset */
+    offset= ud_offset;
+    if (GetConfigROMTaggedRegister(handle, node, 0xD4, &offset, &quadval)!=DC1394_SUCCESS) {
+      *version = -1;
+      return DC1394_FAILURE;
+    }
+    else {
+      udd_offset=(quadval & 0xFFFFFFUL)*4+offset;
+    }
+    
+    if (GetConfigROMTaggedRegister(handle, node, 0x38, &udd_offset, &quadval)!=DC1394_SUCCESS) {
+      // if it fails here we return success with the most accurate version estimation: 1.30.
+      // this is because the GetConfigROMTaggedRegister will return a failure both if there is a comm
+      // problem but also if the tag is not found. In the latter case it simply means that the
+      // camera version is 1.30
+      *version=IIDC_VERSION_1_30;
+      return DC1394_SUCCESS;
+    }
+    else {
+      //fprintf(stderr,"1.3x register is 0x%x\n",quadval);
+      switch (quadval&0xFFFFFFUL) {
+      case 0x10:
+	*version=IIDC_VERSION_1_31;
+	break;
+      case 0x20:
+	*version=IIDC_VERSION_1_32;
+	break;
+      case 0x30:
+	*version=IIDC_VERSION_1_33;
+	break;
+      case 0x40:
+	*version=IIDC_VERSION_1_34;
+	break;
+      case 0x50:
+	*version=IIDC_VERSION_1_35;
+	break;
+      case 0x60:
+	*version=IIDC_VERSION_1_36;
+	break;
+      case 0x70:
+	*version=IIDC_VERSION_1_37;
+	break;
+      case 0x80:
+	*version=IIDC_VERSION_1_38;
+	break;
+      case 0x90:
+	*version=IIDC_VERSION_1_39;
+	break;
+      default:
+	*version=IIDC_VERSION_1_30;
+	break;
+      }
+    }
+  }
+  camera->sw_version=*version;
+  
+  return DC1394_SUCCESS;
 }
 
 void
@@ -1276,8 +1367,12 @@ dc1394_get_camera_misc_info(raw1394handle_t handle, nodeid_t node,
 
     if (dc1394_query_basic_functionality(handle,node,&value) != DC1394_SUCCESS)
         return DC1394_FAILURE;
-    else
-      info->mem_channel_number= (value & 0xF);
+    else {
+      info->mem_channel_number = (value & 0x0000000F);
+      info->bmode_capable      = (value & 0x00800000) != 0;
+      info->one_shot_capable   = (value & 0x00000800) != 0;
+      info->multi_shot_capable = (value & 0x00001000) != 0;
+    }
 
     if (info->mem_channel_number>0) {
       if (dc1394_get_memory_load_ch(handle, node, &info->load_channel)
@@ -1396,6 +1491,9 @@ dc1394_get_camera_feature(raw1394handle_t handle, nodeid_t node,
         feature->trigger_mode= (int)((value >> 14) & 0xF);
         feature->auto_active= DC1394_FALSE;
 	break;
+    case FEATURE_TRIGGER_DELAY:
+        feature->one_push_active= DC1394_FALSE;
+	feature->auto_active=DC1394_FALSE;
     default:
         feature->one_push_active=
             (value & 0x04000000UL) ? DC1394_TRUE : DC1394_FALSE;
@@ -1412,6 +1510,11 @@ dc1394_get_camera_feature(raw1394handle_t handle, nodeid_t node,
       feature->RV_value= value & 0xFFFUL;
       feature->BU_value= (value & 0xFFF000UL) >> 12;
       break;
+    case FEATURE_WHITE_SHADING:
+      feature->R_value=value & 0xFFUL;
+      feature->G_value=(value & 0xFF00UL)>>8;
+      feature->B_value=(value & 0xFF0000UL)>>16;
+      break;
     case FEATURE_TEMPERATURE:
       feature->value= value & 0xFFFUL;
       feature->target_value= value & 0xFFF000UL;
@@ -1502,50 +1605,50 @@ dc1394_print_feature(dc1394_feature_info *f)
     {
         printf("min: %d max %d\n", f->min, f->max);
     }
-
-    if (fid == FEATURE_TRIGGER)
-    {
-        printf("\n\tAvailableTriggerModes: ");
-
-        if (f->trigger_mode_capable_mask & 0x08)
-            printf("0 ");
-        if (f->trigger_mode_capable_mask & 0x04)
-            printf("1 ");
-        if (f->trigger_mode_capable_mask & 0x02)
-            printf("2 ");
-        if (f->trigger_mode_capable_mask & 0x02)
-            printf("3 ");
-        if (!(f->trigger_mode_capable_mask & 0x0f))
-            printf("No modes available");
-
-        printf("\n\tPolarity Change Capable: ");
-
-        if (f->polarity_capable) 
-            printf("True");
-        else 
-            printf("False");
-
-        printf("\n\tCurrent Polarity: ");
-
-        if (f->trigger_polarity) 
-            printf("POS");
-        else 
-            printf("NEG");
-
-        printf("\n\tcurrent mode: %d\n", f->trigger_mode);
-    }
-    else if (fid == FEATURE_WHITE_BALANCE) 
-    {
-        printf("\tB/U value: %d R/V value: %d\n", f->BU_value, f->RV_value);
-    }
-    else if (fid == FEATURE_TEMPERATURE) 
-    {
-        printf("\tTarget temp: %d Current Temp: %d\n", f->target_value,
-               f->value);
-    }
-    else 
-    {
-        printf("\tcurrent value is: %d\n",f->value);
+    switch(fid) {
+    case FEATURE_TRIGGER:
+      printf("\n\tAvailableTriggerModes: ");
+      
+      if (f->trigger_mode_capable_mask & 0x08)
+	printf("0 ");
+      if (f->trigger_mode_capable_mask & 0x04)
+	printf("1 ");
+      if (f->trigger_mode_capable_mask & 0x02)
+	printf("2 ");
+      if (f->trigger_mode_capable_mask & 0x02)
+	printf("3 ");
+      if (!(f->trigger_mode_capable_mask & 0x0f))
+	printf("No modes available");
+      
+      printf("\n\tPolarity Change Capable: ");
+      
+      if (f->polarity_capable) 
+	printf("True");
+      else 
+	printf("False");
+      
+      printf("\n\tCurrent Polarity: ");
+      
+      if (f->trigger_polarity) 
+	printf("POS");
+      else 
+	printf("NEG");
+      
+      printf("\n\tcurrent mode: %d\n", f->trigger_mode);
+      break;
+    case FEATURE_WHITE_BALANCE: 
+      printf("\tB/U value: %d R/V value: %d\n", f->BU_value, f->RV_value);
+      break;
+    case FEATURE_TEMPERATURE:
+      printf("\tTarget temp: %d Current Temp: %d\n", f->target_value, f->value);
+      break;
+    case FEATURE_WHITE_SHADING: 
+      printf("\tR value: %d G value: %d B value: %d\n", f->R_value,
+	     f->G_value, f->B_value);
+      break;
+    default:
+      printf("\tcurrent value is: %d\n",f->value);
+      break;
     }
     if (f->absolute_capable)
       printf("\tabsolute settings:\n\t value: %f\n\t min: %f\n\t max: %f\n",
@@ -1852,31 +1955,112 @@ dc1394_set_video_format(raw1394handle_t handle, nodeid_t node,
     return (retval ? DC1394_FAILURE : DC1394_SUCCESS);
 }
 
+
+int
+dc1394_get_operation_mode(raw1394handle_t handle, nodeid_t node, unsigned int *mode)
+{
+  quadlet_t value_inq, value;
+  int retval;
+
+  retval= dc1394_query_basic_functionality(handle, node, &value_inq);
+  retval= GetCameraControlRegister(handle, node, REG_CAMERA_ISO_DATA, &value);
+  
+  if (value_inq & 0x00800000) {
+    *mode=((value & 0x00008000) >0);
+  }
+  else {
+    *mode=OPERATION_MODE_LEGACY;
+  }
+  
+  return retval;
+}
+
+
+int
+dc1394_set_operation_mode(raw1394handle_t handle, nodeid_t node, unsigned int mode)
+{
+  quadlet_t value_inq, value;
+  int retval;
+  
+  retval= dc1394_query_basic_functionality(handle, node, &value_inq);
+  retval= GetCameraControlRegister(handle, node, REG_CAMERA_ISO_DATA, &value);
+
+  if (mode==OPERATION_MODE_LEGACY) {
+    retval= SetCameraControlRegister(handle, node, REG_CAMERA_ISO_DATA,
+				     (quadlet_t) (value & 0xFFFF7FFF));
+  }
+  else { // 1394b
+    if (value_inq & 0x00800000) { // if 1394b available
+      retval= SetCameraControlRegister(handle, node, REG_CAMERA_ISO_DATA,
+				       (quadlet_t) (value | 0x00008000));
+    }
+    else { // 1394b asked, but it is not available
+      return DC1394_FAILURE;
+    }
+  }
+
+  return (retval ? DC1394_FAILURE : DC1394_SUCCESS);
+  
+}
+
+
 int
 dc1394_get_iso_channel_and_speed(raw1394handle_t handle, nodeid_t node,
                                  unsigned int *channel, unsigned int *speed)
 {
-    quadlet_t value;
-    int retval;
-    
-    retval= GetCameraControlRegister(handle, node, REG_CAMERA_ISO_DATA,
-				     &value);
-    
+  quadlet_t value_inq, value;
+  int retval;
+  
+  retval= dc1394_query_basic_functionality(handle, node, &value_inq);
+  retval= GetCameraControlRegister(handle, node, REG_CAMERA_ISO_DATA, &value);
+  
+  if (value_inq & 0x00800000) { // check if 1394b is available
+    if (value & 0x00008000) { //check if we are now using 1394b
+      *channel= (unsigned int)((value >> 8) & 0x3FUL);
+      *speed= (unsigned int)(value& 0x7UL);
+    }
+    else { // fallback to legacy
+      *channel= (unsigned int)((value >> 28) & 0xFUL);
+      *speed= (unsigned int)((value >> 24) & 0x3UL);
+    }
+  }
+  else { // legacy
     *channel= (unsigned int)((value >> 28) & 0xFUL);
     *speed= (unsigned int)((value >> 24) & 0x3UL);
-
-    return (retval ? DC1394_FAILURE : DC1394_SUCCESS);
+  }
+  
+  return (retval ? DC1394_FAILURE : DC1394_SUCCESS);
 }
 
 int
 dc1394_set_iso_channel_and_speed(raw1394handle_t handle, nodeid_t node,
                                  unsigned int channel, unsigned int speed)
 {
+  quadlet_t value_inq, value;
   int retval;
-  retval= SetCameraControlRegister(handle, node, REG_CAMERA_ISO_DATA,
-				   (quadlet_t) (((channel & 0xFUL) << 28) |
-						((speed & 0x3UL) << 24) ));
-    return (retval ? DC1394_FAILURE : DC1394_SUCCESS);
+
+  retval= dc1394_query_basic_functionality(handle, node, &value_inq);
+  retval= GetCameraControlRegister(handle, node, REG_CAMERA_ISO_DATA, &value);
+  if ((value_inq & 0x00800000)&&(value & 0x00008000)) {
+    // check if 1394b is available and if we are now using 1394b
+    retval= SetCameraControlRegister(handle, node, REG_CAMERA_ISO_DATA,
+				     (quadlet_t) ( ((channel & 0x3FUL) << 8) |
+						   (speed & 0x7UL) |
+						   (0x1 << 15) ));
+  }
+  else { // fallback to legacy
+    if (speed>SPEED_400) {
+      fprintf(stderr,"(%s) line %d: an ISO speed >400Mbps was requested while the camera is in LEGACY mode\n",__FILE__,__LINE__);
+      fprintf(stderr,"              Please set the operation mode to OPERATION_MODE_1394B before asking for\n");
+      fprintf(stderr,"              1394b ISO speeds\n");
+      return DC1394_FAILURE;
+    }
+    retval= SetCameraControlRegister(handle, node, REG_CAMERA_ISO_DATA,
+				     (quadlet_t) (((channel & 0xFUL) << 28) |
+						  ((speed & 0x3UL) << 24) ));
+  }
+  
+  return (retval ? DC1394_FAILURE : DC1394_SUCCESS);
 }
 
 int
@@ -2118,35 +2302,35 @@ int
 dc1394_set_gain(raw1394handle_t handle, nodeid_t node,
                 unsigned int gain)
 {
-    return SetFeatureValue(handle, node, FEATURE_GAIN, gain);
+  return SetFeatureValue(handle, node, FEATURE_GAIN, gain);
 }
 
 int
 dc1394_get_iris(raw1394handle_t handle, nodeid_t node,
                 unsigned int *iris)
 {
-    return GetFeatureValue(handle, node, FEATURE_IRIS, iris);
+  return GetFeatureValue(handle, node, FEATURE_IRIS, iris);
 }
 
 int
 dc1394_set_iris(raw1394handle_t handle, nodeid_t node,
                 unsigned int iris)
 {
-    return SetFeatureValue(handle, node, FEATURE_IRIS, iris);
+  return SetFeatureValue(handle, node, FEATURE_IRIS, iris);
 }
 
 int
 dc1394_get_focus(raw1394handle_t handle, nodeid_t node,
                  unsigned int *focus)
 {
-    return GetFeatureValue(handle, node, FEATURE_FOCUS, focus);
+  return GetFeatureValue(handle, node, FEATURE_FOCUS, focus);
 }
 
 int
 dc1394_set_focus(raw1394handle_t handle, nodeid_t node,
                  unsigned int focus)
 {
-    return SetFeatureValue(handle, node, FEATURE_FOCUS, focus);
+  return SetFeatureValue(handle, node, FEATURE_FOCUS, focus);
 }
 
 int
@@ -2154,43 +2338,104 @@ dc1394_get_temperature(raw1394handle_t handle, nodeid_t node,
                        unsigned int *target_temperature,
                        unsigned int *temperature)
 {
-    quadlet_t value;
-    int retval= GetCameraControlRegister(handle, node,
-                                         REG_CAMERA_TEMPERATURE, &value);
-    *target_temperature= (unsigned int)((value >> 12) & 0xFFF);
-    *temperature= (unsigned int)(value & 0xFFFUL);
-    return (retval ? DC1394_FAILURE : DC1394_SUCCESS);
+  quadlet_t value;
+  int retval= GetCameraControlRegister(handle, node,
+				       REG_CAMERA_TEMPERATURE, &value);
+  *target_temperature= (unsigned int)((value >> 12) & 0xFFF);
+  *temperature= (unsigned int)(value & 0xFFFUL);
+  return (retval ? DC1394_FAILURE : DC1394_SUCCESS);
 }
 
 int
 dc1394_set_temperature(raw1394handle_t handle, nodeid_t node,
                        unsigned int target_temperature)
 {
-    int retval;
-    quadlet_t curval;
+  int retval;
+  quadlet_t curval;
+  
+  if (GetCameraControlRegister(handle, node, REG_CAMERA_TEMPERATURE,
+			       &curval) < 0) {
+    return DC1394_FAILURE;
+  }
+  
+  curval= (curval & 0xFF000FFFUL) | ((target_temperature & 0xFFFUL) << 12);
+  retval= SetCameraControlRegister(handle, node, REG_CAMERA_TEMPERATURE,
+				   curval);
+  return (retval ? DC1394_FAILURE : DC1394_SUCCESS);
+}
 
-    if (GetCameraControlRegister(handle, node, REG_CAMERA_TEMPERATURE,
-                                 &curval) < 0)
-    {
-        return DC1394_FAILURE;
-    }
+int
+dc1394_get_white_shading(raw1394handle_t handle, nodeid_t node,
+                         unsigned int *r_value, unsigned int *g_value, unsigned int *b_value)
+{
+  quadlet_t value;
+  int retval= GetCameraControlRegister(handle, node,
+				       REG_CAMERA_WHITE_SHADING, &value);
+  
+  *r_value= (unsigned int)((value & 0xFF0000UL) >> 16);
+  *g_value= (unsigned int)((value & 0xFF00UL) >> 8);
+  *b_value= (unsigned int)(value & 0xFFUL);
+  return (retval ? DC1394_FAILURE : DC1394_SUCCESS);
+}
 
-    curval= (curval & 0xFF000FFFUL) | ((target_temperature & 0xFFFUL) << 12);
-    retval= SetCameraControlRegister(handle, node, REG_CAMERA_TEMPERATURE,
-                                     curval);
-    return (retval ? DC1394_FAILURE : DC1394_SUCCESS);
+int
+dc1394_set_white_shading(raw1394handle_t handle, nodeid_t node,
+			 unsigned int r_value, unsigned int g_value, unsigned int b_value)
+
+{
+  int retval;
+  quadlet_t curval;
+  
+  if (GetCameraControlRegister(handle, node, REG_CAMERA_WHITE_SHADING,
+			       &curval) < 0)
+    return DC1394_FAILURE;
+  
+  curval= (curval & 0xFF000000UL) | ( ((r_value & 0xFFUL) << 16) |
+				      ((g_value & 0xFFUL) << 8) |
+				      (b_value & 0xFFUL) );
+  retval= SetCameraControlRegister(handle, node, REG_CAMERA_WHITE_SHADING,
+				   curval);
+  return (retval ? DC1394_FAILURE : DC1394_SUCCESS);
+}
+
+int
+dc1394_get_trigger_delay(raw1394handle_t handle, nodeid_t node,
+			 unsigned int *trigger_delay)
+{
+  return GetFeatureValue(handle, node, FEATURE_TRIGGER_DELAY, trigger_delay);
+}
+
+int
+dc1394_set_trigger_delay(raw1394handle_t handle, nodeid_t node,
+			 unsigned int trigger_delay)
+{
+  return SetFeatureValue(handle, node, FEATURE_TRIGGER_DELAY, trigger_delay);
+}
+
+int
+dc1394_get_frame_rate(raw1394handle_t handle, nodeid_t node,
+		      unsigned int *frame_rate)
+{
+  return GetFeatureValue(handle, node, FEATURE_FRAME_RATE, frame_rate);
+}
+
+int
+dc1394_set_frame_rate(raw1394handle_t handle, nodeid_t node,
+		      unsigned int frame_rate)
+{
+  return SetFeatureValue(handle, node, FEATURE_FRAME_RATE, frame_rate);
 }
 
 int
 dc1394_get_trigger_mode(raw1394handle_t handle, nodeid_t node,
                         unsigned int *mode)
 {
-    quadlet_t value;
-    int retval= GetCameraControlRegister(handle, node,
-                                         REG_CAMERA_TRIGGER_MODE, &value);
-
-    *mode= (unsigned int)( ((value >> 16) & 0xFUL) ) + TRIGGER_MODE_MIN;
-    return (retval ? DC1394_FAILURE : DC1394_SUCCESS);
+  quadlet_t value;
+  int retval= GetCameraControlRegister(handle, node,
+				       REG_CAMERA_TRIGGER_MODE, &value);
+  
+  *mode= (unsigned int)( ((value >> 16) & 0xFUL) ) + TRIGGER_MODE_MIN;
+  return (retval ? DC1394_FAILURE : DC1394_SUCCESS);
 }
 
 int
@@ -2892,6 +3137,33 @@ dc1394_get_trigger_on_off(raw1394handle_t handle, nodeid_t node,
   return dc1394_is_feature_on(handle, node, FEATURE_TRIGGER, on_off);
 }
 
+int
+dc1394_set_soft_trigger(raw1394handle_t handle, nodeid_t node)
+{
+  int retval= SetCameraControlRegister(handle, node, REG_CAMERA_SOFT_TRIGGER, ON_VALUE);
+  return (retval ? DC1394_FAILURE : DC1394_SUCCESS);
+}
+
+int
+dc1394_unset_soft_trigger(raw1394handle_t handle, nodeid_t node)
+{
+  int retval= SetCameraControlRegister(handle, node, REG_CAMERA_SOFT_TRIGGER, OFF_VALUE);
+  return (retval ? DC1394_FAILURE : DC1394_SUCCESS);
+}
+
+int
+dc1394_get_soft_trigger(raw1394handle_t handle, nodeid_t node, dc1394bool_t *is_on)
+{
+  quadlet_t value;
+  int retval;
+  
+  //printf("before gst\n");
+  retval = GetCameraControlRegister(handle, node, REG_CAMERA_SOFT_TRIGGER, &value);
+  //printf("after gst %x, %x\n", retval, value);
+  *is_on = value & ON_VALUE;
+  return (retval ? DC1394_FAILURE : DC1394_SUCCESS);
+}
+
 int
 dc1394_query_absolute_control(raw1394handle_t handle, nodeid_t node,
                               unsigned int feature, dc1394bool_t *value)
@@ -3023,7 +3295,10 @@ dc1394_get_bandwidth_usage(raw1394handle_t handle, nodeid_t node, unsigned int *
       return DC1394_FAILURE;
     
     // mutiply by 4 anyway because the best speed is SPEED_400 only
-    *bandwidth = qpp * 4 * (0x1 << (SPEED_400-speed));
+    if (speed>=SPEED_1600)
+      *bandwidth = qpp << (SPEED_1600-speed);
+    else
+      *bandwidth = qpp >> (speed-SPEED_1600);
   }
   else {
     *bandwidth=0;
