@@ -35,10 +35,13 @@ enum
 {
     SPEED_100= 0,
     SPEED_200,
-    SPEED_400
+    SPEED_400,
+    SPEED_800,
+    SPEED_1600,
+    SPEED_3200
 };
 #define SPEED_MIN                   SPEED_100
-#define SPEED_MAX                   SPEED_400
+#define SPEED_MAX                   SPEED_3200
 #define NUM_SPEEDS                  (SPEED_MAX - SPEED_MIN + 1)
 
 /* Enumeration of camera framerates */
@@ -49,10 +52,12 @@ enum
     FRAMERATE_7_5,
     FRAMERATE_15,
     FRAMERATE_30,
-    FRAMERATE_60
+    FRAMERATE_60,
+    FRAMERATE_120,
+    FRAMERATE_240
 };
 #define FRAMERATE_MIN               FRAMERATE_1_875
-#define FRAMERATE_MAX               FRAMERATE_60
+#define FRAMERATE_MAX               FRAMERATE_240
 #define NUM_FRAMERATES              (FRAMERATE_MAX - FRAMERATE_MIN + 1)
 
 /* Enumeration of camera modes for Format_0 */
@@ -135,10 +140,14 @@ enum {
     COLOR_FORMAT7_YUV444,
     COLOR_FORMAT7_RGB8,
     COLOR_FORMAT7_MONO16,
-    COLOR_FORMAT7_RGB16
+    COLOR_FORMAT7_RGB16,
+    COLOR_FORMAT7_MONO16S,
+    COLOR_FORMAT7_RGB16S,
+    COLOR_FORMAT7_RAW8,
+    COLOR_FORMAT7_RAW16
 };
 #define COLOR_FORMAT7_MIN           COLOR_FORMAT7_MONO8
-#define COLOR_FORMAT7_MAX           COLOR_FORMAT7_RGB16
+#define COLOR_FORMAT7_MAX           COLOR_FORMAT7_RAW16
 #define NUM_COLOR_FORMAT7           (COLOR_FORMAT7_MAX - COLOR_FORMAT7_MIN + 1)
 
 /* Enumeration of trigger modes */
@@ -181,7 +190,10 @@ enum
     FEATURE_FOCUS,
     FEATURE_TEMPERATURE,
     FEATURE_TRIGGER,
-    /* 19 reserved features */
+    FEATURE_TRIGGER_DELAY,
+    FEATURE_WHITE_SHADING,
+    FEATURE_FRAME_RATE,
+    /* 16 reserved features */
     FEATURE_ZOOM,
     FEATURE_PAN,
     FEATURE_TILT,
@@ -195,6 +207,46 @@ enum
 #define FEATURE_MAX                 FEATURE_CAPTURE_QUALITY
 #define NUM_FEATURES                (FEATURE_MAX - FEATURE_MIN + 1)
 
+/* Operation modes */
+enum 
+{
+  OPERATION_MODE_LEGACY = 480,
+  OPERATION_MODE_1394B
+};
+
+/* Format 7 sensor layouts*/
+enum
+{
+  COLOR_FILTER_FORMAT7_RGGB = 512,
+  COLOR_FILTER_FORMAT7_GBRG,
+  COLOR_FILTER_FORMAT7_GRBG,
+  COLOR_FILTER_FORMAT7_BGGR
+};
+#define COLOR_FILTER_FORMAT7_MIN                 COLOR_FILTER_FORMAT7_RGGB
+#define COLOR_FILTER_FORMAT7_MAX                 COLOR_FILTER_FORMAT7_BGGR
+#define NUM_COLOR_FILTER_FORMAT7                (COLOR_FILTER_FORMAT7_MAX - COLOR_FILTER_FORMAT7_MIN + 1)
+
+/* IIDC versions*/
+enum
+{
+  IIDC_VERSION_1_04 = 544,
+  IIDC_VERSION_1_20,
+  IIDC_VERSION_PTGREY,
+  IIDC_VERSION_1_30,
+  IIDC_VERSION_1_31,
+  IIDC_VERSION_1_32,
+  IIDC_VERSION_1_33,
+  IIDC_VERSION_1_34,
+  IIDC_VERSION_1_35,
+  IIDC_VERSION_1_36,
+  IIDC_VERSION_1_37,
+  IIDC_VERSION_1_38,
+  IIDC_VERSION_1_39
+};
+#define IIDC_VERSION_MIN                 IIDC_VERSION_1_02
+#define IIDC_VERSION_MAX                 IIDC_VERSION_1_39
+#define NUM_IIDC_VERSION                (IIDC_VERSION_MAX - IIDC_VERSION_MIN + 1)
+
 /* Maximum number of characters in vendor and model strings */
 #define MAX_CHARS                   32
 
@@ -272,6 +324,10 @@ typedef struct __dc1394_misc_info
   int save_channel;
   int load_channel;
 
+  dc1394bool_t bmode_capable;
+  dc1394bool_t one_shot_capable;
+  dc1394bool_t multi_shot_capable;
+
 } dc1394_miscinfo;
 
 typedef struct __dc1394_feature_info_struct 
@@ -296,6 +352,9 @@ typedef struct __dc1394_feature_info_struct
     int value;
     int BU_value;
     int RV_value;
+    int B_value;
+    int R_value;
+    int G_value;
     int target_value;
 
     dc1394bool_t abs_control;
@@ -449,7 +508,7 @@ dc1394_is_camera(raw1394handle_t handle, nodeid_t node, dc1394bool_t *value);
 
 /* Determine the IIDC Specification version */
 int
-dc1394_get_sw_version(raw1394handle_t handle, nodeid_t node, quadlet_t *value);
+dc1394_get_sw_version(raw1394handle_t handle, nodeid_t node, int *version);
 
 /* Get the camera information and print that structure*/
 void 
@@ -510,13 +569,18 @@ int
 dc1394_set_video_format(raw1394handle_t handle, nodeid_t node,
                         unsigned int format);
 int
+dc1394_get_operation_mode(raw1394handle_t handle, nodeid_t node,
+			  unsigned int *mode);
+int
+dc1394_set_operation_mode(raw1394handle_t handle, nodeid_t node,
+			  unsigned int mode);
+int
 dc1394_get_iso_channel_and_speed(raw1394handle_t handle, nodeid_t node,
                                  unsigned int *channel, unsigned int *speed);
 int
 dc1394_set_iso_channel_and_speed(raw1394handle_t handle, nodeid_t node,
                                  unsigned int channel, unsigned int speed);
 
-
 /* Turn camera on or off */
 int
 dc1394_camera_on(raw1394handle_t handle, nodeid_t node);
@@ -632,6 +696,30 @@ int
 dc1394_set_temperature(raw1394handle_t handle, nodeid_t node,
                        unsigned int target_temperature);
 int
+dc1394_get_white_shading(raw1394handle_t handle, nodeid_t node,
+                         unsigned int *r_value, unsigned int *g_value,
+			 unsigned int *b_value);
+int
+dc1394_set_white_shading(raw1394handle_t handle, nodeid_t node,
+			 unsigned int r_value, unsigned int g_value,
+			 unsigned int b_value);
+int
+dc1394_get_trigger_delay(raw1394handle_t handle, nodeid_t node,
+			 unsigned int *trigger_delay);
+int
+dc1394_set_trigger_delay(raw1394handle_t handle, nodeid_t node,
+			 unsigned int trigger_delay);
+int
+dc1394_get_frame_rate(raw1394handle_t handle, nodeid_t node,
+		      unsigned int *frame_rate);
+int
+dc1394_set_frame_rate(raw1394handle_t handle, nodeid_t node,
+		      unsigned int frame_rate);
+  int
+dc1394_get_hue(raw1394handle_t handle, nodeid_t node,
+	       unsigned int *hue);
+
+int
 dc1394_get_trigger_mode(raw1394handle_t handle, nodeid_t node,
                         unsigned int *mode);
 int
@@ -923,7 +1011,18 @@ int
 dc1394_get_trigger_on_off(raw1394handle_t handle, nodeid_t node,
                           dc1394bool_t *on_off);
 
+/* Turn one software trigger on or off and get state */
+int
+dc1394_set_soft_trigger(raw1394handle_t handle, nodeid_t node);
 
+int
+dc1394_unset_soft_trigger(raw1394handle_t handle, nodeid_t node);
+
+int
+dc1394_get_soft_trigger(raw1394handle_t handle, nodeid_t node,
+			dc1394bool_t *is_on);
+
+ 
 /*************************************************
   FORMAT_7 access functions
 **************************************************/
@@ -1111,6 +1210,25 @@ dc1394_query_format7_unit_position(raw1394handle_t handle, nodeid_t node,
 				   unsigned int mode,
 				   unsigned int *horizontal_pos,
 				   unsigned int *vertical_pos);
+int
+dc1394_query_format7_data_depth(raw1394handle_t handle, nodeid_t node,
+				unsigned int mode,
+				unsigned int *data_depth);
+
+int
+dc1394_query_format7_frame_interval(raw1394handle_t handle, nodeid_t node,
+				    unsigned int mode,
+				    float *interval);
+
+int
+dc1394_set_format7_color_filter_id(raw1394handle_t handle, nodeid_t node,
+				   unsigned int mode,
+				   unsigned int color_id);
+
+int
+dc1394_query_format7_color_filter_id(raw1394handle_t handle, nodeid_t node,
+				     unsigned int mode,
+				     unsigned int *color_id);
 
 /**********************************
  *   ABSOLUTE SETTING FUNCTIONS   *		     
