@@ -41,6 +41,9 @@
 #define REG_CAMERA_FORMAT7_BYTE_PER_PACKET               0x044U
 #define REG_CAMERA_FORMAT7_PACKET_PER_FRAME_INQ          0x048U
 #define REG_CAMERA_FORMAT7_UNIT_POSITION_INQ             0x04CU
+#define REG_CAMERA_FORMAT7_FRAME_INTERVAL_INQ            0x050U
+#define REG_CAMERA_FORMAT7_DATA_DEPTH_INQ                0x054U
+#define REG_CAMERA_FORMAT7_COLOR_FILTER_ID               0x058U
 #define REG_CAMERA_FORMAT7_VALUE_SETTING                 0x07CU
 
 /**********************/ 
@@ -220,7 +223,7 @@ _dc1394_v130_handshake(raw1394handle_t handle, nodeid_t node, int mode)
   dc1394_camerahandle *camera;
   camera = (dc1394_camerahandle*) raw1394_get_userdata( handle );
 
-  if (camera->sw_version == 0x102UL) { // if version is 1.30.
+  if (camera->sw_version >= IIDC_VERSION_1_30) {
     // We don't use > because 114 is for ptgrey cameras which are not 1.30 but 1.20
     if (dc1394_query_format7_value_setting(handle, node, mode, &v130handshake,
 					   &setting_1, &err_flag1, &err_flag2)
@@ -273,7 +276,7 @@ _dc1394_v130_errflag2(raw1394handle_t handle, nodeid_t node, int mode)
 
   //fprintf(stderr,"Checking error flags\n");
 
-  if (camera->sw_version == 0x102UL) { // if version is 1.30.
+  if (camera->sw_version >= IIDC_VERSION_1_30) { // if version is 1.30.
     // We don't use > because 0x114 is for ptgrey cameras which are not 1.30 but 1.20
     if (dc1394_query_format7_value_setting(handle, node, mode, &v130handshake,
 					   &setting_1, &err_flag1, &err_flag2)
@@ -321,16 +324,22 @@ _Format7BytePerPixel(int color_coding)
       break;
     case COLOR_FORMAT7_RGB16: return 6.0;
       break;
+    case COLOR_FORMAT7_MONO16S: return 2.0;
+      break;
+    case COLOR_FORMAT7_RGB16S: return 6.0;
+      break;
+    case COLOR_FORMAT7_RAW8: return 1.0;
+      break;
+    case COLOR_FORMAT7_RAW16: return 2.0;
+      break;
     default: return (-1);
     }
 }
 
  
-/*======================================================================*/
-/*! 
+/*====================================================================== 
  *   see documentation of dc1394_setup_format7_capture() in
  *   dc1394_control.h
- *
  *======================================================================*/
 int
 _dc1394_basic_format7_setup(raw1394handle_t handle, nodeid_t node,
@@ -545,7 +554,7 @@ _dc1394_basic_format7_setup(raw1394handle_t handle, nodeid_t node,
    *  ensure that quadlet aligned buffers are big enough, still expect
    *  problems when width*height  != quadlets_per_frame*4
    *-----------------------------------------------------------------------*/
-  if (camerahandle->sw_version == 0x102UL) { // if version is 1.30
+  if (camerahandle->sw_version >= IIDC_VERSION_1_30) { // if version is 1.30
     if (dc1394_query_format7_packet_per_frame(handle, node, mode, &packets_per_frame)!=DC1394_SUCCESS) {
       printf("(%s) Unable to get format 7 packets per frame %d \n", __FILE__, mode);
       return DC1394_FAILURE;
@@ -987,7 +996,7 @@ dc1394_query_format7_value_setting(raw1394handle_t handle, nodeid_t node,
     dc1394_camerahandle *camera;
     camera = (dc1394_camerahandle*) raw1394_get_userdata( handle );
    
-    if (camera->sw_version==0x102UL) {
+    if (camera->sw_version>=IIDC_VERSION_1_30) {
 
       if ( (mode > MODE_FORMAT7_MAX) || (mode < MODE_FORMAT7_MIN) )
 	{
@@ -1053,7 +1062,7 @@ dc1394_query_format7_packet_per_frame(raw1394handle_t handle, nodeid_t node,
     dc1394_camerahandle *camera;
     camera = (dc1394_camerahandle*) raw1394_get_userdata( handle );
    
-    if (camera->sw_version==0x102UL) {
+    if (camera->sw_version>=IIDC_VERSION_1_30) {
 
       if ( (mode > MODE_FORMAT7_MAX) || (mode < MODE_FORMAT7_MIN) )
 	{
@@ -1105,7 +1114,7 @@ dc1394_query_format7_unit_position(raw1394handle_t handle, nodeid_t node,
 	return DC1394_FAILURE;
       }
 
-    if (camera->sw_version==0x102UL) {
+    if (camera->sw_version>=IIDC_VERSION_1_30) {
       retval= GetCameraFormat7Register(handle, node, mode,
 				       REG_CAMERA_FORMAT7_UNIT_POSITION_INQ,
 				       &value);
@@ -1122,3 +1131,74 @@ dc1394_query_format7_unit_position(raw1394handle_t handle, nodeid_t node,
 
     return retval;
 }
+
+int
+dc1394_query_format7_frame_interval(raw1394handle_t handle, nodeid_t node,
+				    unsigned int mode,
+				    float *interval)
+{   
+  quadlet_t value;
+
+  if ( (mode > MODE_FORMAT7_MAX) || (mode < MODE_FORMAT7_MIN) )
+    return DC1394_FAILURE;
+  
+  if (GetCameraFormat7Register(handle, node, mode, REG_CAMERA_FORMAT7_FRAME_INTERVAL_INQ,
+			       &value)==DC1394_FAILURE)
+    return DC1394_FAILURE;
+  
+  *interval=value;
+  
+  return DC1394_SUCCESS;
+}   
+    
+int
+dc1394_query_format7_data_depth(raw1394handle_t handle, nodeid_t node,
+				unsigned int mode,
+				unsigned int *data_depth)
+{   
+  quadlet_t value;
+
+  if ( (mode > MODE_FORMAT7_MAX) || (mode < MODE_FORMAT7_MIN) )
+    return DC1394_FAILURE;
+  
+  if (GetCameraFormat7Register(handle, node, mode, REG_CAMERA_FORMAT7_DATA_DEPTH_INQ,
+			       &value)==DC1394_FAILURE)
+    return DC1394_FAILURE;
+  
+  *data_depth=value >> 24;
+  return DC1394_SUCCESS;
+}   
+    
+int
+dc1394_query_format7_color_filter_id(raw1394handle_t handle, nodeid_t node,
+				     unsigned int mode,
+				     unsigned int *color_id)
+{   
+  quadlet_t value;
+  
+  if ( (mode > MODE_FORMAT7_MAX) || (mode < MODE_FORMAT7_MIN) )
+    return DC1394_FAILURE;
+  
+  if (GetCameraFormat7Register(handle, node, mode, REG_CAMERA_FORMAT7_COLOR_FILTER_ID,
+			       &value)==DC1394_FAILURE)
+    return DC1394_FAILURE;
+  
+  *color_id= (value >> 24)+COLOR_FORMAT7_MIN;
+  return DC1394_SUCCESS;
+}   
+     
+int
+dc1394_set_format7_color_filter_id(raw1394handle_t handle, nodeid_t node,
+				   unsigned int mode,
+				   unsigned int color_id)
+{   
+
+  if ( (mode > MODE_FORMAT7_MAX) || (mode < MODE_FORMAT7_MIN) )
+    return DC1394_FAILURE;
+  
+  if (SetCameraFormat7Register(handle, node, mode, REG_CAMERA_FORMAT7_COLOR_FILTER_ID,
+			       color_id - COLOR_FORMAT7_MIN)==DC1394_FAILURE)
+    return DC1394_FAILURE;
+  
+  return DC1394_SUCCESS;
+}   
